//  Versin para motor HT
//  Modificar la lnea 173 en el programa dependiendo de la relacin de engranes:
//  Para una relacin de 1:34, la frmula es: rpm =  (4 * delta) / 24
//  Para una relacin de 1:54, la frmula es: rpm = (10 * delta) / 99
//  Para una relacin de 1:36, la frmula es: rpm = (10 * delta) / 66
//	Programa ejemplo que muestra en una terminal serial configurada a 57600 bauds,
//	las revoluciones por minuto, el sentido de giro del eje del motor y el avance
//	en ticks (pulsos generador por los encoders), y el ciclo de trabajo que se le
//	aplica al motor, se puede definir el ciclo de trabajo mediante teclear el la
//	terminal serial el comando:
//	
//		pwm=n
//
//	Donde n es un numero entero desde 0 hasta 100, por ejemplo si se requiere aplicar
//	al motor un ciclo de trabajo de 50% se escribe en la terminal serial:
//
//		pwm=50
//
//
//	Para definir el sentido de giro, se usan los comandos:
//
//		horario			el eje del motor girar en el sentido de las manecillas del reloj
//		
//		antihorario		el eje del motor girar en el sentido contraria a las manecillas
//						del reloj
//
//		paro			detiene completamente el giro del eje del motor
//		
//	Debido a la carga que representan los leds conectados al puerto B, se utliza un 
//	circuito integrado transmisor / receptor de datoS 74LS245 conectado de la siguiente 
//	forma:
//															
//																	 Conector 				
//																		J6
//																		
//																	   - RA4
//																	   - RA5
//																	   - RC0
//																	   - RC1
//		  						  _______ _______			+5V--------- +5V
//	  	  ENCODER  		  +5V  --|DIR	 U	  VCC|--  +5V  	GND--------- GND	
//								-|A1			E|--  GND  		
//	 -------M1					-|A2		   B1|-					 Conector	
//	|		GND--- GND			-|A3		   B2|-						J7	
//	|		C1-------------------|A4		   B3|-						 
//	|		C2-------------------|A5		   B4|----------------------B0	 
//	|		3.3V-- +5V			-|A6		   B5|----------------------B1	 
//	|	 ---M2					-|A7		   B6|-						B2
//	|	|						-|A8		   B7|-			 -----------B3	(CCP2)
//	|	|				   GND --|GND		   B8|-			|			B4
//	|	|  						  _______________			|	 -------B5
//	|	|													|	|	 ---B6
//	|	|													|	|	|	B7
//	|	|			   _____________________________		|	|	|	
//	|	|			  |								|		|	|	|	
//	|	 -------------|DRIVER		   ENABLE (PWM) |-------	|	|
//	|				  |DE 					   DIR1 |-----------	|
//	 -----------------|POTENCIA				   DIR2 |---------------
//					  |_____________________________|
//
//
//	El encoder debe entregar seales en cuadratura, pare el caso de este motor se tienen
//	748 ticks o pulsos por revolucin del eje, el motor tiene una relacin de reduccin 
//	de 1:34
//
//

#include <p18cxxx.h>
#include <p18f2550.h>
#include <delays.h>
#include <stdio.h>		//library sprintf()
#include <stdlib.h>		//libreras atoi(),atof()
#include <usart.h>
#include <timers.h>
#include <string.h>
//#include <float.h>
#include <pwm.h>


#define DIR_A	PORTBbits.RB5
#define DIR_B	PORTBbits.RB6
#define GIRO_HORARIO		DIR_A=1; DIR_B=0
#define	GIRO_ANTIHORARIO	DIR_A=0; DIR_B=1
#define MOTOR_DETENIDO		DIR_A=0; DIR_B=0	
#define Horario		0
#define Antihorario	1
#define Detenido	2
#define SW1  	PORTAbits.RA4


// Declaraciones prototipo de funciones 

// Rutina de servicio de interrupcion
void int0_isr(void);  
void timer_isr(void);
void temporizador_0(void);
void comando_serial(void);

void setup(void); 
void delay_ms(int);

// See c018i.c in your C18 compiler dir 
extern void _startup( void ); 


volatile static long avance_anterior=0;	// Util para medir las revoluciones por minuto
volatile static long avance=0;			// Nmero de ticks que ha avanzado el eje del motor

volatile static int rpm=0;
volatile static unsigned int sentido_giro=Horario;	// Define el sentido de giro
volatile static unsigned int int_datos=0;
volatile unsigned static int indice=0;
volatile static int delta=0;

float rpm_f=0.0;

volatile char comando[30];

#pragma code _RESET_INTERRUPT_VECTOR = 0x000800
void _reset( void ) 
{ 
  _asm goto _startup _endasm 
} 

void InterruptServiceHigh(void);

#pragma code InterruptVectorHigh = 0x0808
void InterruptVectorHigh (void)
{
  _asm
    goto InterruptServiceHigh 		//Salta a la direccion de interrupcion ALTA
  _endasm
}

void InterruptServiceLow(void);
#pragma code InterruptVectorLow = 0x0818
void InterruptVectorLow (void)
{
  _asm
    goto InterruptServiceLow //Salta a la direccion de interrupcion BAJA
  _endasm
}



#pragma code

#pragma interrupt InterruptServiceHigh  // Directiva pragma "interrupt" para interrupcion de alta prioridad
void InterruptServiceHigh(void)
{
  	if (INTCONbits.INT0IF)   	//Si la bandera de interrupcin es de INT0 es 1
	{ 
		if(PORTBbits.RB0==PORTBbits.RB1) avance--;
		else avance++;			// Determina el sentido de giro
		INTCON2bits.INTEDG0 = ~INTCON2bits.INTEDG0;	// Cambia el flanco para la proxima interrupcion
		INTCONbits.INT0IF = 0;
  	}                 
}

#pragma interruptlow InterruptServiceLow// Directiva pragma "interruptlow" para interrupcion de baja prioridad
void InterruptServiceLow(void)
{	
	if(INTCONbits.TMR0IF)	temporizador_0();
	if(PIR1bits.RCIF)		comando_serial();
}

void temporizador_0(void)
{
	PORTBbits.RB7 = ~PORTBbits.RB7 ;

	delta= avance - avance_anterior; sentido_giro=Horario;
	if(delta < 0){ delta=~delta; sentido_giro=Antihorario;}
	rpm_f = (10.0 * delta) / 99;				// Determina las revoluciones por minuto cada medio segundo
	if(rpm<0) rpm=~rpm;
	avance_anterior=avance;
	WriteTimer0(42099);						
	INTCONbits.TMR0IF = 0 ;					// Habilita de nuevo la interrupcin
}

void comando_serial(void)
{
	int_datos=1;
	comando[indice++] = ReadUSART();	// Lee el buffer de entrada del puerto serie del pic
	if(indice>=29) indice=0;						
	PIR1bits.RCIF=0;					// Limpia la bandera de interrupcion
}


void main()  
{
	char entero[12];		// Arreglo para representar el avance del eje del motor
	char revoluciones[12];	// Arreglo para representer las revoluciones por minuto

	char valor_pwm[] = "pwm=";
	char comandoHorario[] = "horario";
	char comandoAntih[] = "antihorario";
	char comandoDetenido[] = "paro";
	char valor_pwm_numerico[5];
	int numero_pwm=0;
	unsigned int indice_com=0;
	unsigned char j;
	char pwm_string[8];

	unsigned int ciclo_pwm;

	INTCONbits.INT0IF = 0;		//Bandera de intrrupcion externa a 0
	INTCONbits.TMR0IF = 0;		//Bandera de interrupcin del TIMER0 a cero
	PIR1bits.RCIF=0;			//Limpia la bandera de interrupcion por llegada de datos

	INTCONbits.INT0IE = 1;		//Habilitar interrupcion INT0

	INTCON2bits.INTEDG0 = 1;	//Interrupcin se activa en flanco de subida 
	INTCON2bits.RBPU = 0;		//Deshabilita las reistencias de pullup

	TRISBbits.TRISB0 = 1;		
	TRISBbits.TRISB1 = 1;		//Pines PORTB 0 y 1 como entradas
	TRISBbits.TRISB3 = 0;		//Salida de la seal pwm
	TRISBbits.TRISB5 = 0;
	TRISBbits.TRISB6 = 0;		//Pines PORTB 3, 5 y 6 como salidas
	TRISBbits.TRISB7 = 0;

	WriteTimer0(42099);
	OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_16BIT & T0_PS_1_256);	//Inicia los parmetros del TIMER0
	OpenTimer2(TIMER_INT_OFF & T2_PS_1_16);		//Preescalador a 16 para el timer 2 como fuente de reloj para el generador PWM

	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_ON &
			  USART_ASYNCH_MODE & USART_EIGHT_BIT &
			  USART_CONT_RX & USART_BRGH_LOW, 77);		//SPBRG = 77 PARA 9600 BPS, SPBRG = 12 PARA 57600 BPS 

	INTCON2bits.TMR0IP	= 0;	//Interrupcion TIMER0 de baja prioridad
	IPR1bits.RCIP = 0;			//Interrupcin por llegada de datos es de baja prioridad

	RCONbits.IPEN = 1;			//Habilta las prioridades en las interrupciones

	GIRO_HORARIO;				// en sentido horario
	sentido_giro=Horario;
	
	for(indice=0; indice<30; indice++) comando[indice]=0;

	OpenPWM2(0x95);				//Frecuencia de 5 KHz
	PORTBbits.RB7 = 0;
	SetDCPWM2(420);
	ciclo_pwm=70;

	INTCONbits.GIEH = 1;		//Habilita las interrupciones de alta prioridad
	INTCONbits.GIEL = 1;		//Hablita las inerrupciones de baja prioridad

	INTCONbits.PEIE = 1;		//Habilita las interrupciones de los perifericos
	INTCONbits.GIE = 1;			//Habilitando interrupciones globales

	//delay_ms(100);

	RCREG=0xFF;

	PIR1bits.RCIF=1;

	putrsUSART((const far rom char *)"Control PWM para motor\n\r");
	delay_ms(2000);

	while(1)
	{
			
		if(int_datos)
		{	
			delay_ms(50);
			if(strstr(comando, valor_pwm)) 
			{
				for(j=0; j<7; j++) valor_pwm_numerico[j]=comando[j+4];
				ciclo_pwm=atoi(valor_pwm_numerico);
				if(ciclo_pwm>100 || ciclo_pwm<0)	
				{
					putrsUSART((const far rom char*)"\n\rEl valor pwm debe estar dentro del ranngo 0 a 100\n\r");	
					ciclo_pwm=0;
					SetDCPWM2((ciclo_pwm*600)/100);
					delay_ms(1000);
				}
				else SetDCPWM2((ciclo_pwm*600)/100);
			}
			else if(strstr(comando, comandoAntih))
			{
				GIRO_ANTIHORARIO;			// Giro en sentido antihorario
				sentido_giro=Antihorario;	// Variable para indicar el sentido de giro
			}
			else if(strstr(comando, comandoHorario))
			{
				GIRO_HORARIO;			// Giro en sentido horario
				sentido_giro=Horario;	// Variable para indicar el sentido de giro
			}
			else if(strstr(comando, comandoDetenido))
			{
				MOTOR_DETENIDO;			
			}
			else 
			{	
				putrsUSART((const far rom char*)"\n\rComando no reconocido\n\r");
				delay_ms(1000);
			}
			for(indice=0; indice<30; indice++) comando[indice]=0;
			indice=0; int_datos=0;				
		}

		rpm=rpm_f;
		itoa(rpm,revoluciones); 	// Convierte un entero a cadena de caracteres
		putsUSART(revoluciones); 
		putrsUSART((const far rom char *)" rpm, ");
		
		if(rpm==0) putrsUSART((const far rom char *)"eje detenido, ");
		else if(sentido_giro==Horario) putrsUSART((const far rom char *)"sentido horario, ");
		else putrsUSART((const far rom char *)"sentido antihorario, ");
		ltoa(avance, entero); 
		putsUSART(entero); 
		putrsUSART((const far rom char *)" ticks, ciclo pwm = ");
		ltoa(ciclo_pwm, pwm_string);
		putsUSART(pwm_string);
		putrsUSART((const far rom char *)"\n\r");

		delay_ms(250);
	
	}
} 

void setup()   { 
  //init_18F2550();
  PORTB = 0x01;
  // IPEN esta en 1
  RCONbits.IPEN = 1; 	// Habilita las prioridades de las interrupciones
  INTCONbits.PEIE = 1; 
  // bit PERIPHERAL INTERRUPT ENABLE REGISTER igual a 1
  INTCONbits.GIE = 1; 	// Hablilita las interrupciones globales	
} 

void delay_ms(int i)
{
  int j;
  for(j=0;j<i;j++)
  {
    Delay1KTCYx(12); 	
    //48 MHz, retardo de 1 ms aproximadamente
  }                   
}